/*
 * oglw.c -- $Id$
 * GL window management for MS Windows
 *
 * Copyright (c) 2000.  See accompanying LEGAL file for details.
 */

#include "pstdlib.h"
#include "playw.h"
#include "playgl.h"

static LRESULT CALLBACK w_glsubclass(HWND hwnd,
                                     UINT msg, WPARAM wp, LPARAM lp);

struct p_glwin {
  p_win *parent;
  HWND w;
  HDC dc;
  HGLRC glrc;
  int x, y;
};

static unsigned char bits3[8] = {
  0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0777>>1 };
static unsigned char bits2[4] = { 0, 0x55, 0xaa, 0xff };
static unsigned char bits1[2] = { 0, 0xff };
static unsigned char *bits[3] = { bits1, bits2, bits3 };
static int std_ndx[13] = {
  0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91 };
static PALETTEENTRY std_pal[20] = {
  { 0,   0,   0,    0 },{ 0x80,0,   0,    0 },{ 0,   0x80,0,    0 },
  { 0x80,0x80,0,    0 },{ 0,   0,   0x80, 0 },{ 0x80,0,   0x80, 0 },
  { 0,   0x80,0x80, 0 },{ 0xC0,0xC0,0xC0, 0 },
  { 192, 220, 192,  0 },{ 166, 202, 240,  0 },{ 255, 251, 240,  0 },
  { 160, 160, 164,  0 },
  { 0x80,0x80,0x80, 0 },{ 0xFF,0,   0,    0 },{ 0,   0xFF,0,    0 },
  { 0xFF,0xFF,0,    0 },{ 0,   0,   0xFF, 0 },{ 0xFF,0,   0xFF, 0 },
  { 0,   0xFF,0xFF, 0 },{ 0xFF,0xFF,0xFF, 0 }};

p_glwin *
p_glcreate(p_win *parent, int width, int height, int x, int y)
{
  int is_pixmap = (parent->bm!=0);
  HDC dc = parent->dc;
  p_scr *s = parent->s;
  p_glwin *w = p_malloc(sizeof(p_glwin));
  HWND hw = 0;
  HGLRC glrc = 0;
  PIXELFORMATDESCRIPTOR pfd;
  int pf_index;

  if (!w) return 0;

  if (!is_pixmap) {
    DWORD style = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
    DWORD xstyle = 0;
    hw = CreateWindowEx(xstyle, w_win_class, 0, style,
                       x, y, width, height, parent->w, 0,
                       w_app_instance, 0);
    if (!hw) return 0;
    SetWindowLong(hw, GWL_USERDATA, (LONG)w);
    /* idea -- subclass the window to pass events to parent */
    SetWindowLong(hw, GWL_WNDPROC, (LONG)w_glsubclass);
    ShowWindow(hw, SW_SHOWNA);
    dc = GetDC(hw);
  }

  pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
  pfd.nVersion = 1;
  pfd.dwFlags = (is_pixmap? PFD_DRAW_TO_BITMAP : PFD_DRAW_TO_WINDOW) |
    PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
  pfd.iPixelType = PFD_TYPE_RGBA;
  pfd.cColorBits = 24;
  pfd.cDepthBits = 32;
  pfd.iLayerType = PFD_MAIN_PLANE;
  pfd.cAccumBits = pfd.cStencilBits = pfd.cAuxBuffers = 0;
  /* rest are ignored */
  pfd.cRedBits = pfd.cGreenBits = pfd.cBlueBits = pfd.cAlphaBits = 8;
  pfd.cRedShift = pfd.cGreenShift = pfd.cBlueShift = pfd.cAlphaShift = 0;
  pfd.cAccumRedBits = pfd.cAccumGreenBits = pfd.cAccumBlueBits = 0;
  pfd.cAccumAlphaBits = 0;
  pfd.bReserved = 0;
  pfd.dwLayerMask = pfd.dwVisibleMask = pfd.dwDamageMask = 0;

  pf_index = ChoosePixelFormat(dc, &pfd);
  if (pf_index && SetPixelFormat(dc, pf_index, &pfd)) {
    /* retrieve actual pfd obtained */
    pf_index = DescribePixelFormat(dc, pf_index,
                                  sizeof(PIXELFORMATDESCRIPTOR), &pfd);
    glrc = wglCreateContext(dc);
    if (glrc) {
      w->parent = parent;
      w->w = hw;
      w->dc = dc;
      w->glrc = glrc;
      w->x = x;
      w->y = y;

      if (pfd.dwFlags & PFD_NEED_PALETTE) {
        HPALETTE palette;
        /* from MSDN CUBE sample code */
        unsigned int i, n = 1 << pfd.cColorBits;
        LOGPALETTE *pal = p_malloc(sizeof(LOGPALETTE)+n*sizeof(PALETTEENTRY));
        int r = pfd.cRedBits-1, g = pfd.cGreenBits-1, b = pfd.cBlueBits-1;
        unsigned int rmsk, gmsk, bmsk;
        if (r<0 || r>2) r = 0;
        if (g<0 || g>2) r = 0;
        if (b<0 || b>2) r = 0;
        rmsk = (2<<r)-1, gmsk = (2<<g)-1, bmsk = (2<<b)-1;
        if (n>256) n = 256;
        pal->palVersion = 0x300;
        pal->palNumEntries = n;
        for (i=0 ; i<n ; i++) {
          pal->palPalEntry[i].peRed = bits[r][(i>>pfd.cRedShift)&rmsk];
          pal->palPalEntry[i].peGreen = bits[g][(i>>pfd.cGreenShift)&gmsk];
          pal->palPalEntry[i].peBlue = bits[b][(i>>pfd.cBlueShift)&bmsk];
          pal->palPalEntry[i].peFlags = 0;
        }
        if ((pfd.cColorBits == 8) &&
            (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
            (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
            (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)) {
          for (i=1 ; i<=12 ; i++)
            pal->palPalEntry[std_ndx[i]] = std_pal[i];
        }
        palette = CreatePalette(pal);
        p_free(pal);
        if (palette) {
          HPALETTE pt = SelectPalette(parent->dc, palette, 0);
          if (pt) DeleteObject(pt);
          parent->rgb_mode = 1;  /* inhibit any further palette changes */
          RealizePalette(parent->dc);
          SelectPalette(dc, palette, 0);
          RealizePalette(dc);
        }
      }
    }
  }

  if (!glrc) {
    if (hw) DestroyWindow(hw);
    p_free(w);
    w = 0;
  }

  return w;
}

void
p_gldestroy(p_glwin *w)
{
  if (w) {
    wglDeleteContext(w->glrc);
    if (w->w) DestroyWindow(w->w);
    p_free(w);
  }
}

void
p_glswap(p_glwin *w)
{
  SwapBuffers(w->dc);
  GdiFlush();
}

void
p_glcurrent(p_glwin *w)
{
  wglMakeCurrent(w->dc, w->glrc);
}

static float rot1[3] = {30., 20., -10.};
extern p_glwin *glw1;
extern void init_cube(float *rot);
extern void draw_cube(float *rot);

static LRESULT CALLBACK
w_glsubclass(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
  p_glwin *pgl = (p_glwin *)GetWindowLong(hwnd, GWL_USERDATA);
  if (pgl && pgl->w==hwnd) {
    hwnd = pgl->parent->w;
    switch (msg) {
    case WM_PAINT:
      InvalidateRect(hwnd, 0, 0);
      ValidateRect(pgl->w, 0);
      return 0;
    case WM_DESTROY:
      pgl->w = 0;
      p_gldestroy(pgl);
      return 0;
    case WM_RBUTTONDOWN:
    case WM_MBUTTONDOWN:
    case WM_LBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_MBUTTONUP:
    case WM_LBUTTONUP:
    case WM_MOUSEMOVE:
      {
        int x = LOWORD(lp);
        int y = HIWORD(lp);
        x += pgl->x;
        y += pgl->y;
        lp = MAKELPARAM(x, y);
      }
      break;
    }
  }
  return CallWindowProc(w_winproc, hwnd, msg, wp, lp);
}
